package com.polarsparc.avro;

import java.io.File;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.apache.avro.Schema;
import org.apache.avro.util.Utf8;
import org.apache.avro.io.DatumReader;
import org.apache.avro.io.DatumWriter;
import org.apache.avro.io.Encoder;
import org.apache.avro.io.JsonEncoder;
import org.apache.avro.io.JsonDecoder;
import org.apache.avro.io.BinaryDecoder;
import org.apache.avro.io.EncoderFactory;
import org.apache.avro.io.DecoderFactory;
import org.apache.avro.generic.GenericData;
import org.apache.avro.generic.GenericRecord;
import org.apache.avro.generic.GenericDatumWriter;
import org.apache.avro.generic.GenericDatumReader;

public class Contact {
	public static void main(String[] args) {
		try {
			// Create a schema
			
			Schema schema = new Schema.Parser().parse(new File("resources/Contact.avsc"));
			
			// Create a record from the schema
			
			GenericRecord srec = new GenericData.Record(schema);
			srec.put("First", new Utf8("John"));
			srec.put("Last", new Utf8("Doe"));
			srec.put("Email", new Utf8("john.doe@space.com"));
			srec.put("Mobile", new Utf8("123-456-7890"));
			
			// Create a writer to serialize the record
			
			DatumWriter<GenericRecord> writer = new GenericDatumWriter<GenericRecord>(schema);
			
			// Demonstrate Binary serialization
			
			ByteArrayOutputStream stream = new ByteArrayOutputStream();
			Encoder encoder = EncoderFactory.get().binaryEncoder(stream, null);
			writer.write(srec, encoder);
			encoder.flush();
			stream.close();
			
			// Create raw bytes of the Binary serialized record
			
			byte[] raw1 = stream.toByteArray();
			
			System.out.println("=> Serialized binary data size: " + raw1.length);
			
			// Demonstrate JSON serialization
			
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			JsonEncoder jenc = EncoderFactory.get().jsonEncoder(schema, baos);
			writer.write(srec, jenc);
			jenc.flush();
			baos.close();
			
			System.out.println("=> Serialized json: " + baos);
			System.out.println("=> Serialized json data size: " + baos.toByteArray().length);
			
			// Create raw bytes of the JSON serialized record
			
			byte[] raw2 = baos.toByteArray();
			
			// Create a reader to deserialize the raw bytes
			
			DatumReader<GenericRecord> reader = new GenericDatumReader<GenericRecord>(schema);
			
			// Demonstrate Binary deserialization
			
			BinaryDecoder decoder = DecoderFactory.get().binaryDecoder(raw1, null);
			GenericRecord brec = reader.read(null, decoder);
			
			System.out.printf("=> Binary deserialized record: %s %s - %s\n", brec.get("First"), brec.get("Last"), brec.get("Email"));
			
			// Demonstrate JSON deserialization
			
			ByteArrayInputStream bais = new ByteArrayInputStream(raw2);
			JsonDecoder jdec = DecoderFactory.get().jsonDecoder(schema, bais);
			GenericRecord jrec = reader.read(null, jdec);
			
			System.out.printf("=> Json deserialized record: %s %s - %s\n", jrec.get("First"), jrec.get("Last"), jrec.get("Mobile"));
		}
		catch (Exception ex) {
			ex.printStackTrace(System.out);
		}
	}
}
