package com.polarsparc.avro;

import java.io.File;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.apache.avro.Schema;
import org.apache.avro.util.Utf8;
import org.apache.avro.io.DatumReader;
import org.apache.avro.io.DatumWriter;
import org.apache.avro.io.JsonEncoder;
import org.apache.avro.io.JsonDecoder;
import org.apache.avro.io.EncoderFactory;
import org.apache.avro.io.DecoderFactory;
import org.apache.avro.generic.GenericData;
import org.apache.avro.generic.GenericRecord;
import org.apache.avro.generic.GenericDatumWriter;
import org.apache.avro.generic.GenericDatumReader;

public class SchemaChange {
	public static void main(String[] args) {
		try {
			// Create a schema1 for the serializer
			
			Schema schema1 = new Schema.Parser().parse(new File("resources/Customer.avsc"));
			
			// Create a schema2 for the deserializer
			
			Schema schema2 = new Schema.Parser().parse(new File("resources/Customer-V2.avsc"));
			
			// Create a Customer record from the serializer schema1
			
			GenericRecord srec = new GenericData.Record(schema1);
			srec.put("First", new Utf8("John"));
			srec.put("Last", new Utf8("Doe"));
			GenericRecord ssub = new GenericData.Record(schema1.getField("Contacts").schema());
			ssub.put("Email", new Utf8("john.doe@space.com"));
			ssub.put("Mobile", new Utf8("123-456-7890"));
			srec.put("Contacts", ssub);
			
			// Create a writer to serialize the record
			
			DatumWriter<GenericRecord> writer = new GenericDatumWriter<GenericRecord>(schema1);
			
			// Use JSON serialization using schema1
			
			ByteArrayOutputStream baos1 = new ByteArrayOutputStream();
			JsonEncoder jenc1 = EncoderFactory.get().jsonEncoder(schema1, baos1);
			writer.write(srec, jenc1);
			jenc1.flush();
			baos1.close();
			
			System.out.println("=> Serialized json (V1): " + baos1);
			System.out.println("=> Serialized json (V1) data size: " + baos1.toByteArray().length);
			
			// Create a reader to deserialize the serialized bytes
			
			DatumReader<GenericRecord> reader = new GenericDatumReader<GenericRecord>(schema1, schema2);
			
			// Demonstrate deserialization using schema2
			
			ByteArrayInputStream bais = new ByteArrayInputStream(baos1.toByteArray());
			JsonDecoder jdec = DecoderFactory.get().jsonDecoder(schema1, bais);
			GenericRecord jrec = reader.read(null, jdec);
			
			System.out.printf("=> Json (V2) deserialized record: %s %s - %s, Zip: %s\n", jrec.get("First"), 
				jrec.get("Last"), ((GenericRecord)jrec.get("Contacts")).get("Primary"),
				((GenericRecord)jrec.get("Contacts")).get("Zipcode"));
		}
		catch (Exception ex) {
			ex.printStackTrace(System.out);
		}
	}
}
